/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.hortonworks.spark.sql.hive.llap.common;

import java.util.List;
import java.util.Optional;

/**
 * DescribeTableOutput - To hold output of desc formatted <table> command
 */
public class DescribeTableOutput {

  private List<Column> columns;
  private List<Column> partitionedColumns;
  private List<Column> detailedTableInfoColumns;
  private List<Column> storageInfoColumns;

  //Add more as needed

  public List<Column> getColumns() {
    return columns;
  }

  public void setColumns(List<Column> columns) {
    this.columns = columns;
  }

  public List<Column> getPartitionedColumns() {
    return partitionedColumns;
  }

  public void setPartitionedColumns(List<Column> partitionedColumns) {
    this.partitionedColumns = partitionedColumns;
  }

  public List<Column> getDetailedTableInfoColumns() {
    return detailedTableInfoColumns;
  }

  public List<Column> getStorageInfoColumns() {
    return storageInfoColumns;
  }

  public void setDetailedTableInfoColumns(List<Column> detailedTableInfoColumns) {
    this.detailedTableInfoColumns = detailedTableInfoColumns;
  }

  public void setStorageInfoColumns(List<Column> storageInfoColumns) {
    this.storageInfoColumns = storageInfoColumns;
  }


  public Column findByColNameInStorageInfo(String columnName, boolean errorOnUnsuccessfulSearch) {
    return findByColName(storageInfoColumns, columnName, errorOnUnsuccessfulSearch);
  }

  public Column findByDatatypeNameInStorageInfo(String dataTypeName, boolean errorOnUnsuccessfulSearch) {
    return findByDataTypeName(storageInfoColumns, dataTypeName, errorOnUnsuccessfulSearch);
  }

  private Column findByColName(List<Column> columns, String columnName, boolean errorOnUnsuccessfulSearch) {
    Optional<Column> column = columns.stream().filter(col -> col.getName().equalsIgnoreCase(columnName)).findFirst();
    if (column.isPresent()) {
      return column.get();
    } else {
      if (errorOnUnsuccessfulSearch) {
        throw new IllegalArgumentException("Column with name: "
            + columnName + " cannot be found in describe table output");
      }
    }
    return null;
  }

  private Column findByDataTypeName(List<Column> columns, String dataTypeName, boolean errorOnUnsuccessfulSearch) {
    Optional<Column> column = Optional.empty();
    for (Column col : columns) {
      if (col.getDataType() != null && col.getDataType().equalsIgnoreCase(dataTypeName)) {
        column = Optional.of(col);
        break;
      }
    }
    if (column.isPresent()) {
      return column.get();
    } else {
      if (errorOnUnsuccessfulSearch) {
        throw new IllegalArgumentException("Column with dataType name: "
                + dataTypeName + " cannot be found in describe table output");
      }
    }
    return null;
  }

  @Override
  public String toString() {
    return "DescribeTableOutput{" +
        "columns=" + columns +
        ", partitionedColumns=" + partitionedColumns +
        ", detailedTableInfoColumns=" + detailedTableInfoColumns +
        ", storageInfoColumns=" + storageInfoColumns +
        '}';
  }

}
